;///////////////////////////////////////////////////////////////////////////////////////////////////
;/											Dark Forest	0.9			   							   /
;/								Written by Donald Straney 5/25/03								   /
;/						Designed for the Z80/TI-86 graphing calculator							   /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////DEFINITIONS/////////////////////////////////////////////////

#include "ti86asm.inc"
#include "ti86abs.inc"
#include "ti86ops.inc"

;CALLS

_shracc			equ		4383h			; shift bits in A four to the right
_shlacc			equ		438Bh			; shift bits in A four to the left

;_user_int_ram	equ		$d2fd			;Start of user interrupt routine

;CONSTANTS

Start_Pos		equ		$0000			;Starting position of person on map-y,x
Num_Maps_X		equ		10				;Width of tilemap set
Num_Maps_Y		equ		10				;Height of tilemap set
Delay			equ		51200			;Fade delay coefficient
Contr_Num		equ		180				;Number of keypresses needed to increase contrast
Exit_Num		equ		25				;Number of exit sprite
Back_Num		equ		17				;Number of first background sprite
;Bear_Num		equ		13				;Number of bear
;Bat_Num			equ		12				;Number of bat
;Move_E			equ		200				;Enemy move speed=200/Move_E
;Dir_Left		equ		100				;Numbers used to signal directions between enemy move routines
;Dir_Right		equ		101
;Dir_Up			equ		102
;Dir_Down		equ		103
Map_Start_Addr	equ		$8c00+(40*128)	;Starting map adddress


;VARS

Curr_Pos_X		equ		$ca00			;Current position of person
Curr_Pos_Y		equ		$ca01

Pending_Exit	equ		$ca04			;1 when on exit square

Contr_Orig		equ		$ca05			;User's contrast setting
Contr_Black		equ		$ca06			;Contrast which appears black
Contr_Night		equ		$ca07			;Contrast at which is night in the dark forest

;Move_Dir		equ		$ca0a			;Direction in which to move enemy
;Move_Sprite		equ		$ca0b			;Pointer to enemy sprite to draw

;REAL START OF PROG

.org _asm_exec_ram

	nop									;Display shell title and icon
	jp Prog_Start

.dw 1
.dw ShellTitle
.dw ShellIcon

ShellTitle: .db "Dark Forest by Donald Straney",0	;Description

ShellIcon: .db 8,1
.db %00011000							;Tree sprite
.db %00111100
.db %01111110
.db %00111100
.db %01111110
.db %11111111
.db %00011000
.db %00011000

;/////////////////////////////////////////START OF PROGRAM//////////////////////////////////////////

Prog_Start:								;Start by fading out screen, then fading in title screen
	call _runindicoff					;Turn off that stupid run indicator
	ld a,($c008)						;Save current contrast
	ld (Contr_Orig),a
	add a,6								;Get contrast for night
	ld (Contr_Night),a
	inc a								;Get contrast for blackness
	ld (Contr_Black),a
	jp Back_To_Title					;Now fade out, draw title screen, and fade in again

Title_Key_Loop:
	call _getkey						;Wait for keypress
	cp kF5								;Is f5?
	jp z,_clrLCD						;If so, exit
	cp kExit							;Also exit if exit key is pressed
	jp z,_clrLCD
	cp kF1								;Is f1?
	jr z,Start_Game						;If so, start game
	cp kF2								;Is f2?
	jr z,Show_About						;If so, show about screen
	jr Title_Key_Loop					;Loop until valid key is pressed

Show_About:								;Draw about message
	ld hl,$2000							;Set initial cursor position
	ld (_penCol),hl
Show_About_Loop:						;keypress before exiting
	call Delay_1						;Wait
	ld hl,$fc00							;Draw whole screen in black
	call _invScrn
	ld a,(_penRow)						;Save cursor position, is destroyed by _vputs_ml
	push af
	set textInverse,(iy+textflags)		;Set for inverse draw
	ld hl,About_Msg						;and draw message
	call _vputs_ml
	xor a								;Set cursor back to side of screen
	ld (_penCol),a
	pop af								;Scroll down-restore cursor position
	dec a								;decrement,
	ld (_penRow),a						;and set
	cp 210								;Is at end?
	jr nz,Show_About_Loop				;If not, loop more
End_Show_About:
	res textInverse,(iy+textflags)		;If so, set back to normal text draw
	jp Back_To_Title					;and fade back to title screen

Start_Game:								;Start of game-starting position is already set
										;Map is too large to fit in _asm_exec_ram, so copy it onto page 6
	ld hl,Map_File_Name					;First find the string
	call _MOV10TOOP1
	call _FINDSYM
	ex de,hl
	ld a,b
	push af								;Save its location in memory
	push hl
	call _load_ram_ahl					;find it
	ld e,(hl)							;Get the length word LSB
	pop hl
	pop af
	call _inc_ptr_ahl					;then the MSB
	push af								;Save abs pointer-is destroyed by next routine
	push hl
	call _load_ram_ahl
	ld d,(hl)
	ex de,hl							;Put the length into hl
	xor a								;Set a to 0
	call _SET_MM_NUM_BYTES				;Then set that as number of bytes to copy
	pop hl								;Restore abs pointer
	pop af
	call _inc_ptr_ahl					;increment once to get past length byte
	call _SET_ABS_SRC_ADDR				;and set that as the source address for copy
	ld a,$01							;Now set page 1 as destination address for copy
	ld hl,$0c00
	call _SET_ABS_DEST_ADDR
	call _mm_ldir						;and copy map to page 1
	call _RAM_PAGE_1					;Now load ram page 1 starting at $8000

	call Fade_Out						;Fade to black
	ld hl,$fc00							;Set screen to all black,
	call _invScrn
	ld a,(Contr_Orig)					;then restore contrast, so that message will show up
	ld ($c008),a
	out (2),a
	set textInverse,(iy+textflags)		;Set for inverse draw
	ld hl,$0002							;Position cursor for displaying message 1
	ld (_curRow),hl
	ld hl,Start_Msg						;Display message 1
	call _puts
	ld hl,$0503							;Position cursor for message 2
	ld (_curRow),hl
	ld hl,Start_Msg_2					;Display message 2
	call _puts
	call _pause							;Wait for user to press enter
	res textInverse,(iy+textflags)		;Set back to normal text draw

;Thanks to Matt Johnson at ACZ for the following interrupt install code

;	res 2,(iy+$23)						;turn user int off so it won't get called by
;										;accident before we're ready
;	ld hl,int_copy						;copy prog to user int buffer
;	ld de,_user_int_ram+1				;1 byte ahead of checksum
;	ld bc,int_end-int_start				;length of interrupt handler
;	ldir								;hl is the source, de is the destination,
;										;and bc is the length
;	ld a,(_user_int_ram+1)				;set up checksum byte
;	ld hl,_user_int_ram+($28*1)			;load byte
;	add a,(hl)							;add to total
;	ld hl,_user_int_ram+($28*2)			;load byte
;	add a,(hl)							;add to total
;	ld hl,_user_int_ram+($28*3)			;load byte
;	add a,(hl)							;add to total
;	ld hl,_user_int_ram+($28*4)			;load byte
;	add a,(hl)							;add to total
;	ld hl,_user_int_ram+($28*5)			;load byte
;	add a,(hl)							;add to total
;	ld (_user_int_ram),a				;load checksum in its location
;	set 2,(iy+$23)						;turn it on

	ld hl,Start_Pos						;Set starting position
	ld (Curr_Pos_X),hl
	ld hl,(Curr_Map)					;Get starting tilemap
Set_Map:
	call DrawMap						;Draw tilemap currently pointed to
Move_Person:
	ld hl,(Person_Sprite)				;Draw person at current position
	ld a,(Curr_Pos_X)					;Transfer values from current position memory to
	ld e,a								;registers needed for GridPutSprite
	ld a,(Curr_Pos_Y)
	ld d,a
	ld c,0								;Set for mask so that person will be drawn on top of bridges
	call GridPutSprite					;gates, not overwrite them
Move_Key_Loop:
	call _getkey						;Wait for keypress
	push af								;Save a
	ld hl,Contr_Countr					;Decrease key counter
	dec (hl)
	ld a,(hl)
	call z,Inc_Contr					;If zero, get darker
	jp z,Lose_Game						;If night, lose game
	pop af								;Restore a
	cp kExit							;If exit key,
	jp z,Back_To_Title					;return to title screen
	cp kUp								;If up key,
	jr z,Move_Person_Up					;try to move up
	cp kDown							;If down key,
	jp z,Move_Person_Down				;try to move down
	cp kRight							;If right key,
	jr z,Move_Person_Right				;try to move right
	cp kLeft							;If left key,
	jp z,Move_Person_Left				;try to move left
	jr Move_Key_Loop					;Loop until one of these keys is pressed

Move_Person_Right:
	ld hl,Sprite_Person_R				;Set person sprite to draw
	ld (Person_Sprite),hl
	ld a,(Curr_Pos_X)					;Check to see if is at bottom
	cp $0f
	jp z,Move_Map_Right					;If it is, then go to right map
	call Find_Pixel_Map
	inc hl								;Look at byte to right of person
	call Test_Exit						;If exit, indicate that is on exit tile
	ld a,(hl)							;Test if clear in to-be-moved-to location
	cp Back_Num
	jp c,Move_Key_Loop					;If not, return
	call Clear_Sprite					;If it is clear to the right, then erase person first
	ld hl,Curr_Pos_X					;Move position over 1 unit to right
	inc (hl)
	jp Move_Person						;and move

Move_Person_Up:
	ld hl,Sprite_Person_U				;Set person sprite to draw
	ld (Person_Sprite),hl
	ld a,(Curr_Pos_Y)					;Check to see if is at top
	or a
	jp z,Move_Map_Up					;If it is, then go to top map
	call Find_Pixel_Map
	ld bc,16
	sbc hl,bc							;Look at byte on top of person
	call Test_Exit						;If exit, indicate that is on exit tile
	ld a,(hl)							;Test if clear in to-be-moved-to location
	cp Back_Num
	jp c,Move_Key_Loop					;If not, return
	call Clear_Sprite					;If it is clear on top, then erase person first
	ld hl,Curr_Pos_Y					;Move position 1 up
	dec (hl)
	jp Move_Person						;and move

Move_Person_Left:
	ld hl,Sprite_Person_L				;Set person sprite to draw
	ld (Person_Sprite),hl
	ld a,(Curr_Pos_X)					;Check to see if is at left
	or a
	jp z,Move_Map_Left					;If it is, then go to left map
	call Find_Pixel_Map
	dec hl								;Look at byte to left of person
	call Test_Exit						;If exit, indicate that is on exit tile
	ld a,(hl)							;Test if clear in to-be-moved-to location
	cp Back_Num
	jp c,Move_Key_Loop					;If not, return
	call Clear_Sprite					;If it is clear to the left, then erase person first
	ld hl,Curr_Pos_X					;Move position over 1 unit to left
	dec (hl)
	jp Move_Person						;and move

Move_Person_Down:
	ld hl,Sprite_Person_D				;Set person sprite to draw
	ld (Person_Sprite),hl
	ld a,(Curr_Pos_Y)					;Check to see if is at bottom
	cp 07
	jp z,Move_Map_Down					;If it is, then go to bottom map
	call Find_Pixel_Map
	ld bc,16
	add hl,bc							;Look at sprite below person
	call Test_Exit						;If exit, indicate that is on exit tile
	ld a,(hl)							;Test if clear in to-be-moved-to location
	cp Back_Num
	jp c,Move_Key_Loop					;If not, return
	call Clear_Sprite					;If it is clear on bottom, then erase person first
	ld hl,Curr_Pos_Y					;Move position down 1 unit
	inc (hl)
	jp Move_Person						;and move

Move_Map_Up:							;Draw map on top of current map
	ld a,(Pending_Exit)					;See if was on exit square
	or a
	jp nz,Win_Game						;If it was, show win message
	ld hl,(Curr_Map)					;Get current map
	ld bc,128*Num_Maps_X				;Point to map on top
	sbc hl,bc
;	push hl
;	call Find_Pixel_Map
;	ld a,(hl)
;	cp 06									;(see if position-to-be is clear)
;	pop hl
;	jp c,Move_Key_Loop					;(if not, return)
	ld (Curr_Map),hl					;Otherwise, set it as current map
	ld a,$07							;set position of person
	ld (Curr_Pos_Y),a
	jp Set_Map							;and draw

Move_Map_Down:							;Draw map below current map
	ld a,(Pending_Exit)					;See if was on exit square
	or a
	jp nz,Win_Game						;If it was, show win message
	ld hl,(Curr_Map)					;Get current map
	ld bc,128*Num_Maps_X				;Point to map on bottom
	add hl,bc
;	push hl
;	call Find_Pixel_Map
;	ld a,(hl)
;	cp 06								;(see if position-to-be is clear)
;	pop hl
;	jp c,Move_Key_Loop					;(if not, return)
	ld (Curr_Map),hl					;Otherwise, set it as current map
	xor a								;set position of person
	ld (Curr_Pos_Y),a
	jp Set_Map							;and draw

Move_Map_Left:							;Draw map to left of current map
	ld a,(Pending_Exit)					;See if was on exit square
	or a
	jp nz,Win_Game						;If it was, show win message
	ld hl,(Curr_Map)					;Get current map
	ld bc,128							;Point to previous one
	sbc hl,bc
;	push hl
;	call Find_Pixel_Map
;	ld a,(hl)
;	cp 06									;(see if position-to-be is clear)
;	pop hl
;	jp c,Move_Key_Loop					;(if not, return)
	ld (Curr_Map),hl					;Otherwise, set it as current map
	ld a,$0f							;set position of person
	ld (Curr_Pos_X),a
	jp Set_Map							;and draw

Move_Map_Right:							;Draw map to right of current map
	ld a,(Pending_Exit)					;See if was on exit square
	or a
	jp nz,Win_Game						;If it was, show win message
	ld hl,(Curr_Map)					;Get current map
	ld bc,128							;Point to next one
	add hl,bc
;	push hl
;	call Find_Pixel_Map
;	ld a,(hl)
;	cp 06									;(see if position-to-be is clear)
;	pop hl
;	jp c,Move_Key_Loop					;(if not, return)
	ld (Curr_Map),hl					;Otherwise, set it as current map
	xor a								;set position of person
	ld (Curr_Pos_X),a
	jp Set_Map							;and draw

Win_Game:								;Go here when win
	ld a,(Contr_Orig)					;Set contrast back to normal level to show message
	ld ($c008),a
	out (2),a
	ld hl,$0003							;Position cursor for message 1
	ld (_curRow),hl
	ld hl,Win_Msg						;Display message 1
	call _puts
	ld hl,$0304							;Position cursor for message 2
	ld (_curRow),hl
	ld hl,Win_Msg_2						;Display message 2
	call _puts
	ld hl,$0605							;Position cursor for message 3
	ld (_curRow),hl
	ld hl,Win_Msg_3						;Display message 3
	call _puts
	call _pause							;Wait for user to press enter
	jp Back_To_Title					;then go back to title screen

Lose_Game:
	ld hl,$fc00							;Set whole screen to black
	call _invScrn
	ld a,(Contr_Orig)					;Set contrast back to normal level to show message
	ld ($c008),a
	out (2),a
	set textInverse,(iy+textflags)		;Set for inverse draw
	ld hl,$0201							;Position cursor for message 1
	ld (_curRow),hl
	ld hl,Lose_Msg						;Display message 1
	call _puts
	ld hl,$0102							;Position cursor for message 2
	ld (_curRow),hl
	ld hl,Lose_Msg_2					;Display message 2
	call _puts
	ld a,1
	ld (_curCol),a
	ld hl,Lose_Msg_3					;Display message 3
	call _puts
	ld a,4
	ld (_curCol),a
	ld hl,Lose_Msg_4					;Display message 4
	call _puts
	ld hl,$0606							;Position cursor for message 5
	ld (_curRow),hl
	ld hl,Lose_Msg_5					;Display message 5
	call _puts
	call _pause							;Wait for user to press enter
	res textInverse,(iy+textflags)		;then set back to normal text draw
	jp Back_To_Title					;and go back to title screen

;////////////////////////////////////////////ROUTINES///////////////////////////////////////////////

;============================================================ 
; DrawMap -- draws an 8x16 tilemap	[Assembly Coder's Zenith] 
;	input: HL = pointer to tilemap to draw 
;	total: 34b/91529t (including ret and entire GridPutSprite) 
;============================================================ 
DrawMap:
	ld de,$0000						; start drawing at (0, 0) 
DrawMapL:
	push hl								; save map pointer	
	ld l,(hl)							; get the current tile
	ld h,0								 ; clear the upper byte
	add hl,hl							; multiply by 8 (* 2)
	add hl,hl							; * 4
	add hl,hl							; * 8
	ld bc,Sprites					; load the start of the sprites
	add hl,bc							; add togehter to get correct sprite
	push de								; save sprite draw location
	call GridPutSprite		 ; draw the sprite
	pop de								 ; restore sprite draw location
	pop hl								 ; restore map pointer
	inc hl								 ; go to next tile
	inc e									; move location to the right
	bit 4,e								; if the 5th bit (e = 16), we're done
	jr z,DrawMapL					; only jump if the row isn't complete
	ld e,0								 ; draw at the left again
	inc d									; move location down a row
	bit 3,d								; check if it's done (e = 8)
	jr z,DrawMapL					; jump back to the top of the loop if not done	
	ret									 ; return if we're done
;=========================================================== 
; GridPutSprite:											 [routine by Dan Eble] 
;	Puts an aligned sprite at (E, D), where HL -> Sprite 
;=========================================================== 
; IN : D	= y (0-7 inclusive) 
;			E	= x (0-15 inclusive) 
;			HL-> sprite 
; 
; OUT: AF, BC, DE, HL, IX modified 
; Current total : 28b/567t (not counting ret) 
;=========================================================== 
GridPutSprite:
	push hl
	pop ix					; ix-> sprite
	srl d							; de = 128y+x (16 bytes/row, 8 rows)
	rra
	and $80
	or e					 		; add x offset (remember x <= 15)
	ld e,a
	ld hl,$fc00						; hl-> video ram
	add hl,de							; hl-> video ram + offset
	ld b,8								 ; initialize loop counter
	ld de,$10							; initialize line increment
GPS_Loop:
	ld a,(ix+0)				; get byte from sprite

	push af
	ld a,c			;Get mask flag
	or a
	jr nz,GPS_End_Test	;If it isn't masked, don't mask
	pop af
	or (hl)			;Otherwise, mask
	jr GPS_End_Test_2
GPS_End_Test:
	pop af
GPS_End_Test_2:

	ld (hl),a					; put byte on screen
	inc ix				 		; move to next byte in sprite
	add hl,de		 			; move to next line on screen
	djnz GPS_Loop
	ret 

;===============_vputs_ml by Donald Straney==============
;
;46 bytes
;Inputs: hl-Pointer to zero-terminated string, $d6 is
;		 new line
;		 _penRow-Starting row for text
;		 _penCol-Starting column for text
;
;Outputs: text drawn to screen
;

_vputs_ml:
	push hl								;Save starting position of string
_v_inc_loop:
	ld a,(hl)							;Look at current char in string
	cp $d6								;Is $d6? (new line char)
	jr z,_v_replace_zero				;If so, go to new line
	or a								;Is 0? (end-of-string char)
	jr z,_v_disp_end					;If so, end routine
	inc hl								;Point to next char
	jr _v_inc_loop						;and loop
_v_replace_zero:
	ld (hl),0							;Replace newline char with end-of-string char
	ex de,hl							;Save current position
	pop hl								;and restore beginning-of-line pointer
	push de
	call _vputs							;Draw string
	pop de
	call _v_new_line					;then go to new line
	ex de,hl							;Restore position in string at newline char
	ld (hl),$d6							;(set back to $d6)
	inc hl								;and point to char after $d6
	push hl								;Save that as new beginning-of-line pointer
	jr _v_inc_loop						;Go back to the loop
_v_disp_end:
	pop hl								;Remove beginning-of-line pointer from stack to prevent crash
	jp _vputs							;and draw last line

_v_new_line:							;Move small cursor down 1 row
	ld hl,_penRow
	ld a,6
	add a,(hl)
	ld (hl),a
	xor a
	ld (_penCol),a
	ret

Find_Pixel_Map:							;Find location on current tilemap, return in hl
;	ld a,(Curr_Pos_Y)
;	ld h,a
;	ld a,(Curr_Pos_X)
;	ld l,a

;Find_Pixel_Map_2:						;Same as Find_Pixel_Map, but uses hl for coords
;	ld a,h								;Get aligned y position

	ld a,(Curr_Pos_Y)

	add a,a								;Multiply by 16
	add a,a
	add a,a
	add a,a
	ld e,a								;Put into de
	ld d,0
;	ld a,l								;Now get aligned x position

	ld a,(Curr_Pos_X)

	ld c,a								;and put into bc
	ld b,d
	ld hl,(Curr_Map)					;Now get start of current map
	add hl,bc							;and add offsets
	add hl,de
	ret

;Find_Coord:								;Return coordinates in e,d from Curr_Map and tilemap location
;										;pointer in hl
;	ld de,(Curr_Map)					;Get address of current map in de
;	ex de,hl
;	sbc hl,de							;and subtract from pointer to sprite in map
;										;to get distance of sprite from start of map (top left corner)
;	ld a,l								;Put difference in a
;	call _shracc						;and divide by 16
;	ld d,a								;Set that as the row
;	call _shlacc						;Now shift left again to get row number
;	ld b,a								;and put into b
;	ld a,l								;Now get difference again
;	sub b								;and subtract row from it
;	ld e,a								;to get column number
;	ret

Clear_Sprite:							;Clear sprite at current position in Curr_Pos_X and Y
	call Find_Pixel_Map					;Get address of current position on map
;Clear_Sprite_2:							;Clear sprite at (hl) on tilemap
	ld a,(hl)							;Get sprite at current location
	ld l,a								;Put into hl
	ld h,0
	add hl,hl							;Multiply by 2
	add hl,hl							;*4
	add hl,hl							;*8
	ld de,Sprites						;Add as offset to get sprite address
	add hl,de
	ld a,(Curr_Pos_X)					;Set de as position for GridPutSprite
	ld e,a
	ld a,(Curr_Pos_Y)
	ld d,a
	ld c,1								;Make sure that is NOT masked-sprite has to be cleared!
	jp GridPutSprite					;Draw over current position with sprite at that position on map

Inc_Contr:								;Increase contrast
	ld a,Contr_Num
	ld (Contr_Countr),a
	ld hl,$c008
	inc (hl)
	ld a,(hl)
	out (2),a
	ld hl,Contr_Night
	cp (hl)								;Set zero flag if night
	ret

Test_Exit:								;Test (hl) for exit sprite
	ld b,a								;Save a
	xor a								;Set pending exit to 0
	ld de,Pending_Exit
	ld (de),a
	ld a,(hl)							;Get sprite
	cp Exit_Num							;Is exit?
	jr nz,End_Test_Exit					;If not, end
	ld a,1								;If it is exit, set pending exit
	ld (de),a
End_Test_Exit:
	ld a,b
	ret

_invScrn:								;Sets LCD all black
	ld de,$ffff							;Load de with end-of-screen address
_invScrn_loop:
	ld (hl),$ff							;Set current byte of screen black
	push hl 							;Check to see if at end
	sbc hl,de
	pop hl
	ret z								;If it is, return
	inc hl								;Otherwise, increment pointer
	jr _invScrn_loop					;and loop

Draw_Title:								;Draw title screen
	ld hl,$fd40							;Initialize screen pointer for _invScrn
	call _invScrn						;Set screen black in preparation for title screen draw
	ld hl,Title_Screen					;Copy title screen data
	ld de,$fc00
	ld bc,320
	ldir
	set textInverse,(iy+textflags)		;Set for inverse draw
	ld hl,$2028							;Position cursor
	ld (_penCol),hl
	ld hl,Title_Msg_1					;Draw first message
	call _vputs
	ld hl,$2631							;Position cursor
	ld (_penCol),hl
	ld hl,Title_Msg_2					;Draw second message
	call _vputs
	ld hl,$2c33							;Position cursor
	ld (_penCol),hl
	ld hl,Title_Msg_3					;Draw third message
	call _vputs
	res textInverse,(iy+textflags)		;Set back to normal draw
	ld hl,Sprite_Eyes					;Draw eyes
	ld de,$0502
	ld c,1								;Make sure that is NOT masked
	call GridPutSprite
	ld hl,Sprite_Eyes
	ld de,$030d
	ld c,1
	jp GridPutSprite

Fade_Out:								;Fade out loop
	ld a,($c008)						;Get current contrast
Fade_Out_Loop:
	call Delay_1						;Wait before increasing contrast
	inc a								;When done with delay, increase contrast
	out (2),a
	ld ($c008),a
	ld hl,Contr_Black
	cp (hl)								;Is contrast dark enough?
	jr nz,Fade_Out_Loop					;If not, go through delay and decrease contrast again
	ret									;Otherwise, return

Fade_In:								;Fade in loop
	ld a,($c008)						;Get current contrast
Fade_In_Loop:
	call Delay_1						;Wait
	dec a								;Then decrease contrast
	out (2),a
	ld ($c008),a
	ld hl,Contr_Orig
	cp (hl)								;Is at original contrast?
	jr nz,Fade_In_Loop					;If not, loop
	ret									;If so, return

Delay_1:
	push bc
	ld hl,$0000
	push hl
	pop de
	ld bc,Delay
	ldir
	pop bc
	ret

Back_To_Title:							;Fade out screen, then fade in title screen
;	res 2,(iy+$23)						;Disable user interrupt
	call Fade_Out						;Fade out of current screen
	call Draw_Title						;Draw title screen
	call Fade_In						;Fade in again
	jp Title_Key_Loop					;then respond to keypresses

;Try_Up_Bear:							;Routines for enemy move
;	ld de,Sprite_Bear_Up				;Set to show upwards-moving bear
;	ld (Move_Sprite),de
;	ld a,Dir_Up							;Show that will be moving up
;	ld (Move_Dir),a
;	push hl								;Save pointer to bear's position on tilemap
;	ld bc,16							;Look at next row up
;	sbc hl,bc
;	ld a,(hl)							;and get sprite that's there
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;Try_Up_Bat:
;	ld de,Sprite_Bat					;Set to show bat
;	ld (Move_Sprite),de
;	ld a,Dir_Up							;Show that will be moving up
;	ld (Move_Dir),a
;	push hl								;Save pointer to bat's position on tilemap
;	ld bc,16							;Look at next row up
;	sbc hl,bc
;	ld a,(hl)							;and get sprite that's there
;	sub Back_Num						;Is background?
;	pop hl								;Restore position
;	ret

;Try_Down_Bear:
;	ld de,Sprite_Bear_Down				;Set to show downwards-moving bear
;	ld (Move_Sprite),de
;	ld a,Dir_Down						;Show that will be moving down
;	ld (Move_Dir),a
;	push hl								;Save pointer to bear's position on tilemap
;	ld bc,16							;Look at next row down
;	add hl,bc
;	ld a,(hl)							;and get sprite that's there
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;Try_Down_Bat:
;	ld de,Sprite_Bat					;Set to show downwards-moving bat
;	ld (Move_Sprite),de
;	ld a,Dir_Down						;Show that will be moving down
;	ld (Move_Dir),a
;	push hl								;Save pointer to bat's position on tilemap
;	ld bc,16							;Look at next row down
;	add hl,bc
;	ld a,(hl)							;and get sprite that's there
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;Try_Left_Bear:
;	ld de,Sprite_Bear_Left				;Set to show left-moving bear
;	ld (Move_Sprite),de
;	ld a,Dir_Left						;Show that will be moving left
;	ld (Move_Dir),a
;	push hl								;Save pointer to bear's position on tilemap
;	dec hl								;Look at sprite to the left
;	ld a,(hl)
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;Try_Left_Bat:
;	ld de,Sprite_Bat					;Set to show left-moving bat
;	ld (Move_Sprite),de
;	ld a,Dir_Left						;Show that will be moving left
;	ld (Move_Dir),a
;	push hl								;Save pointer to bat's position on tilemap
;	dec hl								;Look at sprite to the left
;	ld a,(hl)
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;Try_Right_Bear:
;	ld de,Sprite_Bear_Right				;Set to show right-moving bear
;	ld (Move_Sprite),de
;	ld a,Dir_Right						;Show that will be moving right
;	ld (Move_Dir),a
;	push hl								;Save pointer to bear's position on tilemap
;	inc hl								;Look at sprite to the right
;	ld a,(hl)
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;Try_Right_Bat:
;	ld de,Sprite_Bat					;Set to show right-moving bat
;	ld (Move_Sprite),de
;	ld a,Dir_Right						;Show that will be moving right
;	ld (Move_Dir),a
;	push hl								;Save pointer to bat's position on tilemap
;	inc hl								;Look at sprite to the right
;	ld a,(hl)
;	sub Back_Num						;Is background?
;	pop hl
;	ret

;////////////////////////////////////////////DATA/VARS//////////////////////////////////////////////

Contr_Countr: .db Contr_Num				;Keypress counter for increasing contrast

Person_Sprite: .dw Sprite_Person_R		;Current sprite drawn for person

Curr_Map: .dw Map_Start_Addr			;Pointer to current map

Map_File_Name: .db $0c,$08,"ForstMap"	;Name of string with tilemap set

Title_Msg_1: .db "F1-Start game",0		;Title screen messages
Title_Msg_2: .db "F2-About",0
Title_Msg_3: .db "F5-Exit",0

About_Msg:								;About message
.db " Dark Forest",$d6,$d6
.db " Written by Donald Straney",$d6
.db " Thanks to",$d6
.db " ACZ for Draw_Map and",$d6
.db " the interrupt loader code",$d6
.db " Dan Eble for GridPutSprite",$d6
.db " and Ben Weber for the move",$d6
.db " code structure",$d6
.db $d6," Written 5/25/2003",0

Win_Msg: .db "You have escaped the",0	;Win message
Win_Msg_2: .db "Dark Forest and",0
Win_Msg_3: .db "are safe!",0

Lose_Msg: .db "Night has fallen,",0		;Lose message
Lose_Msg_2: .db "and you are still in",0
Lose_Msg_3: .db "the Dark Forest! You",0
Lose_Msg_4: .db "are trapped!",0
Lose_Msg_5: .db "GAME OVER",0

Start_Msg: .db "You have entered the",0	;Message displayed at start of game
Start_Msg_2: .db "Dark Forest",0

Sprite_Eyes:							;Eyes displayed at title screen
	.db %11001100
	.db %11101101
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %00110011
	.db %10110111

Sprite_Person_R:						;Person displayed when moving-facing right
	.db %00000000
	.db %00001000
	.db %00011000
	.db %00111100
	.db %00111100
	.db %00011000
	.db %00001000
	.db %00000000

Sprite_Person_L:						;Person-facing left
	.db %00000000
	.db %00010000
	.db %00011000
	.db %00111100
	.db %00111100
	.db %00011000
	.db %00010000
	.db %00000000

Sprite_Person_U:						;Person-facing up
	.db %00000000
	.db %00000000
	.db %00011000
	.db %01111110
	.db %00111100
	.db %00011000
	.db %00000000
	.db %00000000

Sprite_Person_D:						;Person-facing down
	.db %00000000
	.db %00000000
	.db %00011000
	.db %00111100
	.db %01111110
	.db %00011000
	.db %00000000
	.db %00000000

Sprites:								;Start of sprite data for tilemaps

#include "Sprites.asm"

Title_Screen:							;Title screen data-20 rows-I drew this manually
.db %11000010, %00100001, %00010000, %10001000, %00100100, %00101000, %01001011, %00100100, %00110001, %01000100, %00010100, %11100010, %01100100, %00111001, %00110001, %00100111
.db %10000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000001
.db %11000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000011
.db %10000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000010, %00000001, %00100000, %00000000, %10000001
.db %10001111, %11110000, %00000001, %10000111, %11111000, %00011111, %10111100, %11111111, %11100000, %10111111, %11100111, %11111110, %00011111, %00111111, %11111111, %10000001
.db %11000011, %11011000, %00000011, %11100001, %11000100, %00000110, %00110000, %00111100, %00100001, %11001111, %01110001, %11000010, %00100001, %00100001, %11100000, %10000001
.db %10000001, %10000100, %00000111, %01110000, %10000011, %00000010, %01000000, %00011000, %00000011, %11000110, %00011000, %10000000, %01100000, %00000000, %11000000, %00000001
.db %10000001, %10000010, %00001100, %00110000, %10000011, %10000010, %11000000, %00010000, %00000110, %00100100, %00001000, %10000000, %01110000, %00000000, %10000000, %00000011
.db %10000001, %00000010, %00010000, %00010000, %10000011, %10000011, %10000000, %00011000, %00000100, %00100100, %00001000, %10000000, %01100000, %00000000, %11000000, %00000001
.db %10000001, %00000010, %00011001, %00111000, %10000011, %00000011, %00000000, %00011100, %00100100, %00100100, %00001000, %11001000, %00100000, %00000000, %10000000, %00000011
.db %11000001, %00000010, %00111111, %11111000, %11000100, %00000011, %10000000, %00011111, %11100100, %00100110, %00110000, %11111000, %00011100, %00000000, %10000000, %00000001
.db %10000001, %00000010, %00110100, %01011000, %11111000, %00000011, %11000000, %00011000, %00100100, %00100111, %01100000, %11001000, %00000010, %00000001, %10000000, %00000001
.db %11000001, %10000010, %01100000, %00001000, %11100110, %00000010, %01000000, %00011000, %00000100, %00100111, %11100000, %10000000, %00000001, %10000000, %10000000, %00000001
.db %10000001, %10000010, %01100000, %00001100, %11000011, %00000010, %00100000, %00011000, %00000100, %00100110, %10000000, %10000000, %00000011, %10000000, %11000000, %00000011
.db %10000001, %10000100, %01000000, %00000100, %10000001, %00000010, %00010000, %00011000, %00000110, %00100100, %01000000, %10000000, %00000001, %10000000, %10000000, %00000001
.db %10000011, %11011000, %11000000, %00001101, %11000001, %10000111, %00001000, %00111000, %00000011, %11001100, %01100001, %11000010, %01000010, %00000001, %11000000, %00000001
.db %11000111, %11110001, %11100000, %00111111, %11110111, %11011111, %10011111, %11111110, %00000001, %10111111, %11111111, %11111110, %01111100, %00000111, %11111000, %0000011
.db %10000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000010, %01000000, %00000000, %00000000, %00000001
.db %11000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000000, %00000011
.db %11100100, %01101100, %10000111, %01100001, %00001000, %10001100, %00100110, %01100001, %11000101, %01100100, %01000110, %00011100, %10011000, %00101001, %00100010, %10101111

;int_copy:								;START OF USER INTERRUPT ROUTINE
;
;.org _user_int_ram
;
;int_start:								;User interrupt routine-executes at 200 Hz
;	ld hl,Int_Countr					;Get enemy move rate counter
;	dec (hl)							;and decrement
;	ret nz								;If it's not at 0 yet, then return
;	exx									;Save all registers
;	ld a,Move_E							;Otherwise, reset counter and go on to move code
;	ld (Int_Countr),a
;Scan_Map:								;Find any enemies in current map
;	ld hl,(Curr_Map)					;Get current map
;	ld b,128							;Load loop counter
;Map_Loop:
;	ld a,(hl)							;Get sprite number from tilemap
;	cp Bear_Num							;Is bear?
;	call z,Move_Bear
;	cp Bat_Num							;Is bat?
;	call z,Move_Bat
;	inc hl								;Point to next sprite
;	djnz Map_Loop						;Go until have scanned all of tilemap
;	exx
;	ret									;then return
;
;	nop
;
;Move_Bear:								;Move bear
;	push hl
;	call Try_Down_Bear					;Try to move down first
;	jr nc,Move_Now						;If can move, then move
;	call Try_Left_Bear					;Next try left
;	jr nc,Move_Now
;	call Try_Right_Bear					;then right
;	jr nc,Move_Now
;	call Try_Up_Bear					;what about up?
;	jr nc,Move_Now
;	pop hl
;	ret									;If none of these, then some idiot has trapped the poor bear, don't move at all
;
;	nop
;
;Move_Bat:								;Move bat
;	push hl
;	call Try_Left_Bat					;Try left
;	jr nc,Move_Now
;	call Try_Right_Bat					;then right
;	jr nc,Move_Now
;	call Try_Up_Bat						;then up
;	jr nc,Move_Now
;	call Try_Down_Bat					;then down
;	jr nc,Move_Now						;If can move, then move
;	pop hl
;	ret									;If none, is trapped, return
;
;	nop
;
;Move_Now:								;Now actually move bear
;	call Find_Coord						;Get its onscreen position
;	push de								;Save that
;	ex de,hl							;and put into hl for Find_Pixel_Map
;	call Find_Pixel_Map_2				;Find sprite that was in that location
;	call Clear_Sprite_2					;and draw over it
;	pop de								;Restore the bear's original position
;	ld a,(Move_Dir)						;Look at the direction the bear's supposed to be moved
;	cp Dir_Left							;Is left?
;	call z,Move_Pos_Left
;	cp Dir_Right						;Is right?
;	call z,Move_Pos_Right
;	cp Dir_Up							;Is up?
;	call z,Move_Pos_Up
;	cp Dir_Down							;Is down?
;	call z,Move_Pos_Down
;	ld hl,(Move_Sprite)					;Now get sprite that is supposed to be drawn
;	call GridPutSprite					;and draw it at modified position
;	pop hl
;	ret
;
;	nop
;
;Move_Pos_Left:
;	dec e
;	ret
;
;	nop
;
;Move_Pos_Right:
;	inc e
;	ret
;
;	nop
;
;Move_Pos_Up:
;	dec d
;	ret
;
;	nop
;
;Move_Pos_Down:
;	inc d
;	ret
;
;Int_Countr: .db Move_E					;Enemy move rate counter
;
;int_end:								;END OF USER INTERRUPT

.end